@@ -13,5 +13,5 @@ globalVariables(c("id", "label_en", "new_id", "has_broader", "external",
                   "extLabel", "dist", "dist_0", "dist_1", "dist_2", "label.x",
                   "label.y", "label_harm", "label_new", "has_2_differences",
                   "has_1_difference", "has_0_differences", "has_new_close_match",
-                  "lvl"
+                  "lvl", "dataseries", "id_new", "rest"
                   ))
